{-# LANGUAGE NoImplicitPrelude #-}
module Import
  ( module RIO
  , module Types
  , module Data.Aeson
  , module Data.Default
  , module Text.Show.Pretty
  , module IBClient.Types
  , module System.Directory
  , ppShow'
  , getCurrentDay
  , switchAccountTo
  ) where

import RIO
import Types
import Data.Aeson (FromJSON, ToJSON)
import Data.Default
import Text.Show.Pretty
import IBClient.Types
import System.Directory


--- imports not reexported
import Data.Text as T
import Data.Time.Clock
import Data.Time.Calendar

ppShow' :: Show a => a -> Text
ppShow' = T.pack . ppShow

getCurrentDay :: IO Day
getCurrentDay = utctDay <$> getCurrentTime

switchAccountTo :: Text -> RIO App ()
switchAccountTo a = do
    refs' <- appRefs <$> ask
    let sendQ = twsConnectionSend $ twsConnectionRefs refs'
    -- cancel subscription of old account (if any)
    readTVarIO (currentAccount refs') >>= \case
      Nothing   -> return ()
      Just aid  -> liftIO $ atomically $ writeTQueue sendQ $ Msg_IB_OUT $ IB_RequestAccountData False aid
    -- subscribe to new account
    liftIO $ atomically $ writeTQueue sendQ $ Msg_IB_OUT $ IB_RequestAccountData True a
    -- finally change
    liftIO $ atomically $ modifyTVar' (currentAccount refs') (const $ Just a)
